<?php
include_once(__DIR__ . '/vdevshopify/vdevshopify.app.class.php');

include_once(__DIR__ . '/vdevshopify/vdevshopify.product.class.php');
include_once(__DIR__ . '/vdevshopify/vdevshopify.collection.class.php');
include_once(__DIR__ . '/vdevshopify/vdevshopify.order.class.php');
include_once(__DIR__ . '/vdevshopify/vdevshopify.payment.class.php');
include_once(__DIR__ . '/vdevshopify/vdevshopify.theme.class.php');

class VDEVShopify
{
    public $host;
    
    public $controller;
    public $action;

    public $devMode;
    public $apiData;
    
    private $accessTokenFile;
    
    public $lastCall = [];
    
    public function __construct(array $apiData, $devMode = false)
    {
        $this->host = $apiData['host'];
        $this->apiData = $apiData;
        $this->devMode = $devMode;
        
        if (!empty($_GET['DEV_MODE'])) {
            $this->devMode = true;
        }
        
        $this->controller = isset($_GET['controller']) ? $_GET['controller'] : null;
        $this->action = isset($_GET['action']) ? $_GET['action'] : null;
        
        $this->accessTokenFile = __DIR__ . '/../tmp/access_token.txt';
        $this->apiData['access_token'] = $this->getAccessToken();
    }

    public function getAccessToken()
    {
        if (!empty($_GET['access_token'])) {
            $accessToken = $_GET['access_token'];
            return $accessToken;
        }
        
        // if(is_file($this->accessTokenFile)){
        //     $accessToken = file_get_contents($this->accessTokenFile);
            
        //     if (!empty($accessToken)) {
        //         return $accessToken;
        //     }
        // }
            
        return false;
    }
    
    public function setAccessToken($accessToken) {
        
        if (!empty($accessToken)) {
            
            if(!is_file($this->accessTokenFile)){
                file_put_contents($this->accessTokenFile, $accessToken);
            
            } else {
                $accessTokenContent = file_get_contents($this->accessTokenFile);
                
                if (empty($accessTokenContent)) {
                    file_put_contents($this->accessTokenFile, $accessToken);
                }
            }
        }
    }

    public function apiClass($apiClass)
    {
        switch ($apiClass) {

            case 'product':
                return new VDEVShopifyProduct($this->apiData, $this->devMode);

            case 'collection':
                return new VDEVShopifyCollection($this->apiData, $this->devMode);

            case 'order':
                return new VDEVShopifyOrder($this->apiData, $this->devMode);

            case 'app':
                return new VDEVShopifyApp($this->apiData, $this->devMode);

            case 'payment':
                return new VDEVShopifyPayment($this->apiData, $this->devMode);

            case 'theme':
                return new VDEVShopifyTheme($this->apiData, $this->devMode);

            default:
                return false;
        }
    }

    public function call($uri, array $query = [], $method = 'GET', array $requestHeaders = [], $host = null)
    {
        if (!$host) {
            $host = $this->apiData['shop'];
        }
        
        $url = 'https://' . $host . $uri;

        if (!empty($query) && in_array($method, ['GET', 'DELETE'])) {
            $url .= '?' . http_build_query($query);
        }

        if ($this->devMode) {
            echo '<div style="text-align: center; font-style: italic;">API URL: ' . $url . ' ' . $method . '</div>';
        }

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_HEADER, TRUE);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, TRUE);
        curl_setopt($curl, CURLOPT_MAXREDIRS, 3);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, FALSE);
        // curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 3);
        // curl_setopt($curl, CURLOPT_SSLVERSION, 3);
        curl_setopt($curl, CURLOPT_USERAGENT, 'My New Shopify App v.1');
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        // curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

        /* Setup headers */
        $requestHeaders[] = "";

        if (!is_null($this->getAccessToken())) {
            $requestHeaders[] = "X-Shopify-Access-Token: " . $this->getAccessToken();
        }

        if ($method != 'GET' && in_array($method, ['POST', 'PUT'])) {

            if (is_array($query)) {
                $requestHeaders[] = "Content-Type: application/json";
                $query = json_encode($query);
            }

            curl_setopt($curl, CURLOPT_POSTFIELDS, $query);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, $requestHeaders);


        /* Pagination */
        $GLOBALS['TEMP_provider_shopify_page_info'] = null;
        
        curl_setopt(
            $curl,
            CURLOPT_HEADERFUNCTION,
            function($curl, $headerLine) {
                
                if (preg_match('#^link\: .+#i', $headerLine, $headerLinePreg)) {
                    $headerLineNew = trim(preg_replace('#^link: #i', '', $headerLine));
                    $headerLineParts = explode(', ', $headerLineNew);
                    
                    if (count($headerLineParts) == 2) {
                        $GLOBALS['TEMP_provider_shopify_prev_page_info'] = preg_replace('#^\<.+page\_info\=([A-z0-9\-]+)\>.+$#', '$1', $headerLineParts[0]);
                        $GLOBALS['TEMP_provider_shopify_next_page_info'] = preg_replace('#^\<.+page\_info\=([A-z0-9\-]+)\>.+$#', '$1', $headerLineParts[1]);
                        
                    } else if (preg_match('#^\<[^\>]+page\_info\=([A-z0-9]+)\>\; rel\=\"next\"#i', $headerLineNew, $headerLinePreg)) {
                        $GLOBALS['TEMP_provider_shopify_prev_page_info'] = null;
                        $GLOBALS['TEMP_provider_shopify_next_page_info'] = $headerLinePreg[1];
                    }
                }
                
                return strlen($headerLine);
            }
        );
        /* /Pagination */


        /* Send request to Shopify and capture any errors */
        $response = curl_exec($curl);

        $error_number = curl_errno($curl);
        $error_message = curl_error($curl);

        /* Close cURL to be nice */
        curl_close($curl);

        /* Return an error is cURL has a problem */
        if ($error_number) {
            return $error_message;

        } else {
            /* No error, return Shopify's response by parsing out the body and the headers */
            $response = preg_split("/\r\n\r\n|\n\n|\r\r/", $response, 2);

            /* Convert headers into an array */
            $headers = [];
            $header_data = explode("\n", $response[0]);
            $headers['status'] = $header_data[0]; /* Does not contain a key, have to explicitly set */
            array_shift($header_data); /* Remove status, we've already set it above */

            foreach ($header_data as $part) {
                $h = explode(":", $part);
                $headers[trim($h[0])] = trim($h[1]);
            }
            
            $call = [
                'headers' => $headers,
                'response' => $response[1]
            ];
            
            $this->lastCall = $call;

            return $call;
        }
    }
    
    public function getLastCallError() {
        $response = json_decode($this->lastCall['response'], true);
        
        $error = null;
        
        if (!empty($response['errors'])) {
            $error = json_encode($response['errors']);
        }
        
        return $error;
    }

    public function getCall($call)
    {
        $response = json_decode($call['response'], true);
    }

    public function getCallResponse($call, $responseKey, $pagination = false)
    {
        if (!empty($call['response'])) {
            $response = json_decode($call['response'], true);
    
            if (!empty($response[$responseKey])) {
                $response = $response[$responseKey];
            }
            
            if ($pagination) {
                return [
                    'items' => $response,
                    'pagination' => [
                        'next_page_info' => !empty($GLOBALS['TEMP_provider_shopify_next_page_info']) ? $GLOBALS['TEMP_provider_shopify_next_page_info'] : null,
                        'prev_page_info' => !empty($GLOBALS['TEMP_provider_shopify_prev_page_info']) ? $GLOBALS['TEMP_provider_shopify_prev_page_info'] : null
                    ]
                ];
            }
            
            return $response;
        }

        return false;
    }
}
